define([
    'jquery',
    'mage/translate'
], function ($, $t){
    'use strict';

    $.widget('sync.progress', {
        progress: null,
        message: null,
        totalCount: 0,
        currentCount: 0,

        _create: function(){
            this.initializeProgress();

            this.process(
                this.options.sync.exportids_endpoint,
                this.options.sync.export_endpoint
            );
        },

        initializeProgress: function(){
            let self = this;

            this.progress = $(this.options.sync.selector);
            this.progress.parent().css({
                'border': '1px solid #CCC',
                'padding': '1px'
            });
            this.progress.css({
                'background-color': '#060',
                'height': '25px'
            });

            this.message = $(this.options.sync.message_selector);
        },

        process: function(exportIdsEnpoint, exportEndpoint){
            $.ajax({
                type: 'GET',
                url: exportIdsEnpoint,
                dataType: 'json'
            }).done((data) => {
                this.sync(data, exportEndpoint);
            });
        },

        sync: function(exportIds, exportEndpoint, spinnerText){
            let totalCount = exportIds.length;
            if (totalCount === 0) {
                this.goBack();
            } else {
                this.totalCount = exportIds.length;

                let syncSuccess = 0;
                let syncFailure = 0;
                let syncSkip = 0;

                const endpoint = exportEndpoint;

                const asyncCall = (exportData, index) => {
                    $.ajax({
                        type: 'POST',
                        url: endpoint,
                        data: {
                            ids: [exportData.id],
                            total: totalCount,
                            counter: index,
                            success: syncSuccess,
                            failure: syncFailure
                        },
                        dataType: 'json'
                    }).done((data) => {
                        if (data['success']) {
                            syncSuccess = syncSuccess + data['success'];
                            this.setMessage('"' + exportData.name + '" synchronized...');
                        } else if (data['skip']) {
                            syncSkip = syncSkip + data['skip'];
                            this.setMessage('"' + exportData.name + '" skipped...');
                        } else {
                            syncFailure = syncSuccess + data['err'];
                            this.setMessage('"' + exportData.name + '" not synchronized...', true);
                        }

                        this.currentCount = index;

                        task.next();
                    });
                }

                let self = this;

                function* runner() {
                    let index = 0;
                    while (typeof exportIds[index] !== typeof undefined) {
                        yield asyncCall(exportIds[index], index);
                        self.renderProgress();

                        index++;
                    }

                    self.goBack();
                }

                let task = runner();
                task.next();
            }
        },

        renderProgress: function(){
            this.progress.css({
                'width': ((this.currentCount / this.totalCount) * 100) + '%'
            });
        },

        setMessage: function(message, error){
            if (error) {
                message = '<span style="color:#600;">' + message + '</span>';
            }
            this.message.html('<strong>(' + this.currentCount + ' / ' + this.totalCount + ')</strong> ' + '<span style="color:#060;">' + message + '</span>');
        },

        goBack: function(){
            window.location.href = this.options.return_url;
        }
    });

    return $.sync.progress;
})